@@ -56,8 +56,8 @@ trans_network <- R6Class(classname = "trans_network",
 			if(!is.null(env_cols) | !is.null(add_data)){
 				dataset1$sample_table %<>% .[rownames(.) %in% rownames(env_data), ]
 				dataset1$tidy_dataset(main_data = TRUE)
-				env_data %<>% .[rownames(dataset1$sample_table), ]
-				env_data <- dropallfactors(env_data, unfac2num = TRUE)
+				env_data %<>% .[rownames(dataset1$sample_table), ] %>%
+					dropallfactors(unfac2num = TRUE)
 				env_data[] <- lapply(env_data, function(x){if(is.character(x)) as.factor(x) else x})
 				env_data[] <- lapply(env_data, as.numeric)
 				self$env_data <- env_data
@@ -69,10 +69,11 @@ trans_network <- R6Class(classname = "trans_network",
 			sampleinfo <- dataset1$sample_table
 			# store taxonomic table for the following analysis
 			self$use_tax <- dataset1$tax_table
-			use_abund <- dataset1$otu_table
-			use_abund <- use_abund[apply(use_abund, 1, sum)/sum(use_abund) > filter_thres, ]
+			use_abund <- dataset1$otu_table %>% 
+				{.[apply(., 1, sum)/sum(.) > filter_thres, ]} %>%
+				t %>%
+				as.data.frame
 			
-			use_abund <- as.data.frame(t(use_abund))
 			if( (!is.na(cal_cor)) & (!is.null(env_cols) | !is.null(add_data))){
 				use_abund <- cbind.data.frame(use_abund, env_data)
 			}
@@ -198,11 +199,10 @@ trans_network <- R6Class(classname = "trans_network",
 				}
 			}
 			if(grepl("SpiecEasi", network_method, ignore.case = TRUE)){
-				if(!require(SpiecEasi)){
-					stop("SpiecEasi package not installed")
+				if(!require("SpiecEasi")){
+					stop("SpiecEasi package is not installed! See https://github.com/zdk123/SpiecEasi ")
 				}
-				use_abund <- self$use_abund
-				use_abund <- as.matrix(use_abund)
+				use_abund <- self$use_abund %>% as.matrix
 				# calculate SpiecEasi network, reference https://github.com/zdk123/SpiecEasi
 				network <- spiec.easi(use_abund, method = SpiecEasi_method, ...)
 				network <- adj2igraph(getRefit(network))
@@ -241,8 +241,8 @@ trans_network <- R6Class(classname = "trans_network",
 				close(openfile)
 				system("julia calculate_network.jl")
 				setwd('..')
-				message("The temporary files are in ", tem_dir)
-				network <- read_graph(paste0(tem_dir, "/network_PGM.gml"), format = "gml")
+				message("The temporary files are in ", tem_dir, " ...")
+				network <- read_graph(file.path(tem_dir, "network_PGM.gml"), format = "gml")
 				network <- set_vertex_attr(network, "name", value = V(network)$label)
 				E(network)$label <- unlist(lapply(E(network)$weight, function(x) ifelse(x > 0, "+", "-")))
 				E(network)$weight <- abs(E(network)$weight)
@@ -323,7 +323,7 @@ trans_network <- R6Class(classname = "trans_network",
 		#' @param filepath default "network.gexf"; file path.
 		#' @return None.
 		save_network = function(filepath = "network.gexf"){
-			if(!require(rgexf)){
+			if(!require("rgexf")){
 				stop("Please install rgexf package")
 			}
 			private$check_igraph()
@@ -541,7 +541,7 @@ trans_network <- R6Class(classname = "trans_network",
 		cal_powerlaw_p = function(...){
 			network <- self$res_network
 			degree_dis <- igraph::degree(network)
-			if(!require(poweRlaw)){
+			if(!require("poweRlaw")){
 				stop("Please first install poweRlaw package from CRAN !")
 			}
 			resdispl <- poweRlaw::displ$new(degree_dis + 1)
@@ -597,7 +597,7 @@ trans_network <- R6Class(classname = "trans_network",
 		),
 	private = list(
 		check_igraph = function(){
-			if(!require(igraph)){
+			if(!require("igraph")){
 				stop("Please first install igraph package!")
 			}
 		},
@@ -652,7 +652,7 @@ trans_network <- R6Class(classname = "trans_network",
 			nns
 		},
 		saveAsGEXF = function(network, filepath = "network.gexf"){
-			require(rgexf)
+			require("rgexf")
 			nodes <- data.frame(cbind(V(network), V(network)$name))
 			edges <- get.edges(network, 1:ecount(network))
 			vAttrNames <- setdiff(list.vertex.attributes(network), "name")
@@ -815,7 +815,7 @@ trans_network <- R6Class(classname = "trans_network",
 			if(is.null(roles_colors)){roles_colors <- rev(c("grey80", RColorBrewer::brewer.pal(3, "Dark2")))}
 
 			p <- ggplot() + geom_rect(data=NULL, mapping=aes(xmin=x1, xmax=x2, ymin=y1, ymax=y2, fill=lab, alpha = .4))
-			p <- p + guides(fill=guide_legend(title="Roles"), alpha = FALSE)
+			p <- p + guides(fill=guide_legend(title="Roles"), alpha = "none")
 			p <- p + scale_fill_manual(values = roles_colors)
 			if(module == T){
 				p <- p + geom_point(data=node_roles, aes(x=p, y=z, shape= module)) + theme_bw() + guides(shape=guide_legend(title="Module"))
@@ -869,10 +869,10 @@ trans_network <- R6Class(classname = "trans_network",
 				theme(axis.text.x = element_text(angle = 40, colour = "black", vjust = 1, hjust = 1, size = 10))
 			
 			if(plot_color == use_level){
-				p <- p + guides(color = FALSE)
+				p <- p + guides(color = "none")
 			}
 			if(plot_shape == use_level){
-				p <- p + guides(shape = FALSE)
+				p <- p + guides(shape = "none")
 			}
 			if(!is.null(plot_size)){
 				p <- p + guides(size = guide_legend(title = "Abundance(%)"))
