@@ -10,8 +10,10 @@ trans_venn <- R6Class(classname = "trans_venn",
 	public = list(
 		#' @param dataset the object of \code{\link{microtable}} Class.
 		#' @param sample_names default NULL; if provided, filter the samples.
-		#' @param ratio default numratio; NULL, "numratio" or "seqratio"; numratio: calculate number percentage; seqratio: calculate sequence percentage; 
+		#' @param ratio default NULL; NULL, "numratio" or "seqratio"; numratio: calculate number percentage; seqratio: calculate sequence percentage; 
 		#' 	 NULL: no additional percentage.
+		#' @param add_abund_table default NULL; data.frame or matrix format; additional data provided instead of dataset$otu_table;
+		#' Features must be rows.
 		#' @return venn_table and venn_count_abund stored in trans_venn object.
 		#' @examples
 		#' \donttest{
@@ -19,16 +21,29 @@ trans_venn <- R6Class(classname = "trans_venn",
 		#' t1 <- dataset$merge_samples(use_group = "Group")
 		#' t1 <- trans_venn$new(dataset = t1, ratio = "numratio")
 		#' }
-		initialize = function(dataset = NULL, sample_names = NULL, ratio = "numratio"
+		initialize = function(dataset = NULL, sample_names = NULL, ratio = NULL, add_abund_table = NULL
 			){
-			# first clone the dataset
-			use_dataset <- clone(dataset)
-			if(!is.null(sample_names)){
-				use_dataset$sample_table %<>% .[rownames(.) %in% sample_names, ]
+			if(!is.null(dataset)){
+				# first clone the dataset
+				use_dataset <- clone(dataset)
+				if(!is.null(sample_names)){
+					use_dataset$sample_table %<>% .[rownames(.) %in% sample_names, ]
+				}
+				use_dataset$tax_table %<>% base::subset(use_dataset$taxa_sums() != 0)
+				use_dataset$tidy_dataset()
+				abund <- use_dataset$otu_table
+				self$tax_table <- use_dataset$tax_table
+			}else{
+				if(is.null(add_abund_table)){
+					stop("Either dataset or add_abund_table should be provided !")
+				}else{
+					if(! any(is.data.frame(add_abund_table), is.matrix(add_abund_table))){
+						stop("add_abund_table must be data.frame or matrix !")
+					}
+					abund <- add_abund_table
+				}
 			}
-			use_dataset$tax_table %<>% base::subset(use_dataset$taxa_sums() != 0)
-			use_dataset$tidy_dataset()
-			abund <- use_dataset$otu_table
+
 			col_names <- colnames(abund)
 			colnumber <- ncol(abund)
 			abund1 <- cbind.data.frame(OTU = rownames(abund), abund)
@@ -72,7 +87,6 @@ trans_venn <- R6Class(classname = "trans_venn",
 			self$col_names <- col_names
 			self$ratio <- ratio
 			self$otu_table <- abund
-			self$tax_table <- use_dataset$tax_table
 			message('The result is stored in object$venn_table and object$venn_count_abund ...')
 		},
 		#' @description
@@ -303,13 +317,13 @@ trans_venn <- R6Class(classname = "trans_venn",
 		#' @description
 		#' Transform venn result for the composition analysis.
 		#'
-		#' @param use_OTUs_frequency default TRUE; whether only use OTUs occurrence frequency, i.e. presence/absence data; if FALSE, use abundance data.
+		#' @param use_frequency default TRUE; whether only use OTUs occurrence frequency, i.e. presence/absence data; if FALSE, use abundance data.
 		#' @return a new \code{\link{microtable}} class.
 		#' @examples
 		#' \donttest{
-		#' t2 <- t1$trans_venn_com(use_OTUs_frequency = TRUE)
+		#' t2 <- t1$trans_venn_com(use_frequency = TRUE)
 		#' }
-		trans_venn_com = function(use_OTUs_frequency = TRUE){
+		trans_venn_com = function(use_frequency = TRUE){
 			otudata <- self$otu_table
 			venn_table <- self$venn_table
 			sampledata <- data.frame(SampleID = colnames(venn_table), Group = colnames(venn_table)) %>% 'rownames<-'(colnames(venn_table))
@@ -326,7 +340,7 @@ trans_venn <- R6Class(classname = "trans_venn",
 			tt[is.na(tt)] <- 0
 			tt %<>% 'rownames<-'(.[, 1]) %>% .[, -1, drop = FALSE]
 			colnames(tt) <- colnames(venn_table)
-			if(use_OTUs_frequency == T){
+			if(use_frequency == T){
 				tt[tt != 0] <- 1
 			}
 			microtable$new(sample_table = sampledata, otu_table = tt, tax_table = taxdata)
