@@ -93,3 +93,18 @@ summarySE_inter = function(usedata=NULL, measurevar, groupvars=NULL, na.rm=TRUE)
 	datac
 }
 
+#' A color palette for 20 elements.
+#'
+#' This is one palette option for users who have <= 20 elements to plot.
+#' The palletes of RColorBrewer package provide at most 12 discrete colors, such as "Set3" and "Paired".
+#' This palette is adapted from D3.js library and can be used as a supplement.
+#'
+#' @export
+color_palette_20 <- c("#1f77b4", "#aec7e8", "#ff7f0e", "#ffbb78", "#2ca02c", "#98df8a", "#d62728", "#ff9896", "#9467bd", "#c5b0d5", 
+	"#8c564b", "#c49c94", "#e377c2", "#f7b6d2", "#7f7f7f", "#c7c7c7", "#bcbd22", "#dbdb8d", "#17becf", "#9edae5")
+
+
+
+
+
+
